@@ -365,7 +365,7 @@ impl Executable<'_, '_> {
     /// # Errors
     ///
     /// See [`Error`].
-    pub async fn execute_on_qvm<V: qvm::Client>(&mut self, client: &V) -> ExecutionResult {
+    pub async fn execute_on_qvm<V: qvm::Client + ?Sized>(&mut self, client: &V) -> ExecutionResult {
         #[cfg(feature = "tracing")]
         tracing::debug!(
             num_shots = %self.shots,
@@ -587,6 +587,20 @@ impl<'execution> Executable<'_, 'execution> {
         Ok(job_handle)
     }
 
+    /// Cancel a job that has yet to begin executing.
+    ///
+    /// This action is *not* atomic, and will attempt to cancel a job even if it cannot be cancelled. A
+    /// job can be cancelled only if it has not yet started executing.
+    ///
+    /// Success response indicates only that the request was received. Cancellation is not guaranteed,
+    /// as it is based on job state at the time of cancellation, and is completed on a best effort
+    /// basis.
+    pub async fn cancel_qpu_job(&mut self, job_handle: JobHandle<'execution>) -> Result<(), Error> {
+        let quantum_processor_id = job_handle.quantum_processor_id.to_string();
+        let qpu = self.qpu_for_id(quantum_processor_id).await?;
+        Ok(qpu.cancel_job(job_handle).await?)
+    }
+
     /// Wait for the results of a job submitted via [`Executable::submit_to_qpu`] to complete.
     ///
     /// # Errors
@@ -701,7 +715,7 @@ impl From<ExecutionError> for Error {
             ExecutionError::Unexpected(inner) => Self::Unexpected(format!("{inner:?}")),
             ExecutionError::Quilc { .. } => Self::Connection(Service::Quilc),
             ExecutionError::QcsClient(v) => Self::Unexpected(format!("{v:?}")),
-            ExecutionError::IsaError(v) => Self::Unexpected(format!("{v:?}")),
+            ExecutionError::Isa(v) => Self::Unexpected(format!("{v:?}")),
             ExecutionError::ReadoutParse(v) => Self::Unexpected(format!("{v:?}")),
             ExecutionError::Quil(e) => Self::Quil(e),
             ExecutionError::ToQuil(e) => Self::ToQuil(e),
@@ -709,7 +723,7 @@ impl From<ExecutionError> for Error {
             ExecutionError::RewriteArithmetic(e) => Self::RewriteArithmetic(e),
             ExecutionError::RpcqClient(e) => Self::Unexpected(format!("{e:?}")),
             ExecutionError::Substitution(message) => Self::Substitution(message),
-            ExecutionError::QpuApiError(e) => Self::QpuApiError(e),
+            ExecutionError::QpuApi(e) => Self::QpuApiError(e),
         }
     }
 }
@@ -817,6 +831,7 @@ mod describe_get_config {
 #[cfg(test)]
 #[cfg(feature = "manual-tests")]
 mod describe_qpu_for_id {
+    use assert2::let_assert;
     use std::num::NonZeroU16;
 
     use crate::compiler::quilc::CompilerOpts;
@@ -899,7 +914,8 @@ mod describe_qpu_for_id {
         let mut exe = exe.with_qcs_client(Qcs::default());
         let result = exe.qpu_for_id("Aspen-8").await;
 
-        assert!(matches!(result, Err(_)));
-        assert!(matches!(exe.qpu, None));
+        let_assert!(Err(crate::executable::Error::Unexpected(err)) = result);
+        assert!(err.contains("NoRefreshToken"));
+        assert!(exe.qpu.is_none());
     }
 }
