@@ -16,7 +16,8 @@ use qcs_api_client_grpc::{
     },
     services::controller::{
         controller_client::ControllerClient, execute_controller_job_request,
-        get_controller_job_results_request, ExecuteControllerJobRequest,
+        get_controller_job_results_request, CancelControllerJobsRequest,
+        ExecuteControllerJobRequest, ExecutionOptions as InnerApiExecutionOptions,
         GetControllerJobResultsRequest,
     },
 };
@@ -104,6 +105,7 @@ pub async fn submit(
         execution_configurations: vec![params_into_job_execution_configuration(patch_values)],
         job: Some(execute_controller_job_request::Job::Encrypted(program)),
         target: execution_options.get_job_target(quantum_processor_id),
+        options: execution_options.api_options().cloned(),
     };
 
     let mut controller_client = execution_options
@@ -124,6 +126,79 @@ pub async fn submit(
         .ok_or_else(|| GrpcClientError::ResponseEmpty("Job Execution ID".into()))?)
 }
 
+/// Cancel all given jobs that have yet to begin executing.
+///
+/// This action is *not* atomic, and will attempt to cancel every job even when some jobs cannot be
+/// cancelled. A job can be cancelled only if it has not yet started executing.
+///
+/// Success response indicates only that the request was received. Cancellation is not guaranteed,
+/// as it is based on job state at the time of cancellation, and is completed on a best effort
+/// basis.
+///
+/// # Arguments
+/// * `quantum_processor_id` - The quantum processor to execute the job on. This parameter
+///      is required unless using [`ConnectionStrategy::EndpointId`] in `execution_options`
+///      to target a specific endpoint ID.
+/// * `job_ids` - The [`JobId`]s to cancel.
+/// * `client` - The [`Qcs`] client to use.
+/// * `execution_options` - The [`ExecutionOptions`] to use. If the connection strategy used
+///       is [`ConnectionStrategy::EndpointId`] then direct access to that endpoint
+///       overrides the `quantum_processor_id` parameter.
+pub async fn cancel_jobs(
+    job_ids: Vec<JobId>,
+    quantum_processor_id: Option<&str>,
+    client: &Qcs,
+    execution_options: &ExecutionOptions,
+) -> Result<(), QpuApiError> {
+    let request = CancelControllerJobsRequest {
+        job_ids: job_ids.into_iter().map(|id| id.0).collect(),
+    };
+
+    let mut controller_client = execution_options
+        .get_controller_client(client, quantum_processor_id)
+        .await?;
+
+    controller_client
+        .cancel_controller_jobs(request)
+        .await
+        .map_err(GrpcClientError::RequestFailed)?;
+
+    Ok(())
+}
+
+/// Cancel a job that has yet to begin executing.
+///
+/// This action is *not* atomic, and will attempt to cancel a job even if it cannot be cancelled. A
+/// job can be cancelled only if it has not yet started executing.
+///
+/// Success response indicates only that the request was received. Cancellation is not guaranteed,
+/// as it is based on job state at the time of cancellation, and is completed on a best effort
+/// basis.
+///
+/// # Arguments
+/// * `quantum_processor_id` - The quantum processor to execute the job on. This parameter is
+///      required unless using [`ConnectionStrategy::EndpointId`] in `execution_options` to target
+///      a specific endpoint ID.
+/// * `job_ids` - The [`JobId`]s to cancel.
+/// * `client` - The [`Qcs`] client to use.
+/// * `execution_options` - The [`ExecutionOptions`] to use. If the connection strategy used is
+///      [`ConnectionStrategy::EndpointId`] then direct access to that endpoint overrides the
+///      `quantum_processor_id` parameter.
+pub async fn cancel_job(
+    job_id: JobId,
+    quantum_processor_id: Option<&str>,
+    client: &Qcs,
+    execution_options: &ExecutionOptions,
+) -> Result<(), QpuApiError> {
+    cancel_jobs(
+        vec![job_id],
+        quantum_processor_id,
+        client,
+        execution_options,
+    )
+    .await
+}
+
 /// Fetch results from QPU job execution.
 ///
 /// # Arguments
@@ -194,7 +269,7 @@ pub type QpuConnectionOptionsBuilder = ExecutionOptionsBuilder;
 ///
 /// Use [`Default`] to get a reasonable set of defaults, or start with [`ExecutionOptionsBuilder`]
 /// to build a custom set of options.
-#[derive(Builder, Clone, Debug, Default, PartialEq, Eq)]
+#[derive(Builder, Clone, Debug, Default, PartialEq)]
 pub struct ExecutionOptions {
     #[doc = "The [`ConnectionStrategy`] to use to establish a connection to the QPU."]
     #[builder(default)]
@@ -202,6 +277,61 @@ pub struct ExecutionOptions {
     #[doc = "The timeout to use for the request, defaults to 30 seconds. If set to `None`, then there is no timeout."]
     #[builder(default = "Some(Duration::from_secs(30))")]
     timeout: Option<Duration>,
+    #[doc = "Options avaialable when executing a job on a QPU, particular to the execution service's API."]
+    #[builder(default = "None")]
+    api_options: Option<InnerApiExecutionOptions>,
+}
+
+impl Eq for ExecutionOptions {}
+
+/// Options avaialable when executing a job on a QPU, particular to the execution service's API.
+/// This is a conventent alias for [`InnerApiExecutionOptions`] which provides a builder.
+///
+/// Use [`Default`] to get a reasonable set of defaults, or start with [`ApiExecutionOptionsBuilder`]
+/// to build a custom set of options.
+#[derive(Builder, Clone, Debug, Default, PartialEq)]
+#[allow(clippy::module_name_repetitions)]
+pub struct ApiExecutionOptions {
+    /// the inner proto representation
+    inner: InnerApiExecutionOptions,
+}
+
+impl Eq for ApiExecutionOptions {}
+
+impl ApiExecutionOptions {
+    /// Get an [`ExecutionOptionsBuilder`] that can be used to build a custom [`ExecutionOptions`].
+    #[must_use]
+    pub fn builder() -> ApiExecutionOptionsBuilder {
+        ApiExecutionOptionsBuilder::default()
+    }
+
+    /// Get the configured `bypass_settings_protection` value.
+    #[must_use]
+    pub fn bypass_settings_protection(&self) -> bool {
+        self.inner.bypass_settings_protection
+    }
+}
+
+impl From<ApiExecutionOptions> for InnerApiExecutionOptions {
+    fn from(options: ApiExecutionOptions) -> Self {
+        options.inner
+    }
+}
+
+impl From<InnerApiExecutionOptions> for ApiExecutionOptions {
+    fn from(inner: InnerApiExecutionOptions) -> Self {
+        Self { inner }
+    }
+}
+
+impl ApiExecutionOptionsBuilder {
+    /// Set the `bypass_settings_protection` value.
+    pub fn bypass_settings_protection(&mut self, bypass_settings_protection: bool) -> &mut Self {
+        self.inner
+            .get_or_insert(InnerApiExecutionOptions::default())
+            .bypass_settings_protection = bypass_settings_protection;
+        self
+    }
 }
 
 impl ExecutionOptions {
@@ -222,6 +352,12 @@ impl ExecutionOptions {
     pub fn timeout(&self) -> Option<Duration> {
         self.timeout
     }
+
+    /// Get the [`ApiExecutionOptions`].
+    #[must_use]
+    pub fn api_options(&self) -> Option<&InnerApiExecutionOptions> {
+        self.api_options.as_ref()
+    }
 }
 
 /// The connection strategy to use when submitting and retrieving jobs from a QPU.
