@@ -52,7 +52,7 @@ pub(crate) enum Error {
     #[error("Problem using QCS API: {0}")]
     QcsClient(#[from] GrpcClientError),
     #[error("Problem fetching ISA: {0}")]
-    IsaError(#[from] GetIsaError),
+    Isa(#[from] GetIsaError),
     #[error("Problem parsing memory readout: {0}")]
     ReadoutParse(#[from] MemoryReferenceParseError),
     #[error("Problem when compiling program: {details}")]
@@ -64,7 +64,7 @@ pub(crate) enum Error {
     #[error("Program when getting substitutions for program: {0}")]
     Substitution(String),
     #[error("Problem making a request to the QPU: {0}")]
-    QpuApiError(#[from] super::api::QpuApiError),
+    QpuApi(#[from] super::api::QpuApiError),
 }
 
 impl From<quilc::Error> for Error {
@@ -259,6 +259,17 @@ impl<'a> Execution<'a> {
         ))
     }
 
+    pub(crate) async fn cancel_job(&self, job_handle: JobHandle<'a>) -> Result<(), Error> {
+        crate::qpu::api::cancel_job(
+            job_handle.job_id(),
+            Some(job_handle.quantum_processor_id()),
+            self.client.as_ref(),
+            job_handle.execution_options(),
+        )
+        .await
+        .map_err(Error::from)
+    }
+
     pub(crate) async fn retrieve_results(
         &self,
         job_handle: JobHandle<'a>,
